require 'spec_helper'

class RD3::RepoTest
  include RD3::Model
end

class RD3::RepoTestRepository
  include RD3::Repository

  config_file_name :mysql
end

class RD3::RepoTest2Repository
  include RD3::Repository

  config_file_name :postgresql_search_db
end

class RD3::NoConfigFileRepository
  include RD3::Repository
end

describe RD3::Repository do

  let(:mysql_repo) {RD3::RepoTestRepository}
  let(:postgresql_repo) {RD3::RepoTest2Repository}
  let(:no_config_file_repo) {RD3::NoConfigFileRepository}

  it 'should set and get config file name' do
    mysql_repo.config_file_name.should eq(:mysql)
    postgresql_repo.config_file_name.should eq(:postgresql_search_db)
  end

  it 'should derive data provider from config file name' do
    #mysql_repo.data_provider.should eq(RD3::DataProviders::Rdbms)
    #postgresql_repo.data_provider.should eq(RD3::DataProviders::Rdbms)
    mysql_repo.data_provider.should eq(:rdbms)
    postgresql_repo.data_provider.should eq(:rdbms)
  end

  it 'should derive model from repository name' do
    mysql_repo.model.should eq(RD3::RepoTest)
  end

  context 'exceptions' do

    it 'should raise type error for unsupported data provider' do
      expect {no_config_file_repo.config_file_name :unsupported_config_file}.to raise_error TypeError
    end

    it 'should raise not implemented error for find_by_key' do
      expect {no_config_file_repo.find_by_key(1, nil)}.to raise_error NotImplementedError
    end

    it 'should raise not implemented error for save' do
      expect {no_config_file_repo.save(nil)}.to raise_error NotImplementedError
    end

    it 'should raise not implemented error for delete' do
      expect {no_config_file_repo.delete(1, nil)}.to raise_error NotImplementedError
    end

  end

end
